package net.lensfrex.socket.client;

import javafx.application.Application;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.stage.Stage;

import javafx.scene.control.*;
import javafx.scene.layout.*;
import net.lensfrex.socket.client.controller.ClientConnection;
import net.lensfrex.socket.common.Common;
import net.lensfrex.socket.common.Constant;
import net.lensfrex.socket.common.DataPack;
import net.lensfrex.socket.common.MessageData;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.nio.charset.StandardCharsets;
import java.util.Random;

public class Client extends Application {
    public void start() {
        launch();
    }

    @Override
    public void start(Stage stage) throws Exception {
        Scene scene = new Scene(new UI().buildUI(), 515, 509);

        // 主题，不想要或者网络出错的话注释掉就好了
//        scene.getStylesheets().add(new URL("https://www.lensfrex.net/light.css").toExternalForm());
//        scene.getStylesheets().add(new URL("https://www.lensfrex.net/dark.css").toExternalForm());
        scene.getStylesheets().add(new URL("http://127.0.0.1/light.css").toExternalForm());

        stage.setTitle("(￣﹃￣)");
        stage.setScene(scene);

        stage.show();
    }
}

class UI {
    private static final String[] DEFAULT_USERNAMES = {"Rex", "Red", "Ash", "Aether"};

    private Pane pane;
    private VBox rootBox;

    private HBox serverControlVBox;
    private TextField serverTextField;

    private TextArea messageTextArea;
    private HBox userControlVBox;
    private TextField usernameTextField;
    private Button loginButton, logoutButton;
    private HBox messageControlVBox;
    private TextField messageTextField;
    private Button sendButton;

    protected Parent buildUI() {
        pane = new Pane();

        rootBox = new VBox();
        rootBox.setPadding(new Insets(10, 10, 10, 10));
        rootBox.setSpacing(20);
        rootBox.setPrefWidth(500);
        rootBox.setPrefHeight(500);

        messageTextArea = new TextArea("暂无消息...");
        messageTextArea.setPrefHeight(700);
        messageTextArea.setEditable(false);

        rootBox.getChildren().addAll(serverControlPane(), userControlPane(), messageTextArea, messageControlPane());
        pane.getChildren().addAll(rootBox);

        return pane;
    }

    private HBox serverControlPane() {
        serverControlVBox = new HBox();
        serverControlVBox.setSpacing(10);
        serverControlVBox.setAlignment(Pos.CENTER_LEFT);

        Label serverLabel = new Label("服务器地址：");

        serverTextField = new TextField("127.0.0.1:2333");
        serverTextField.setPrefWidth(320);

        serverControlVBox.getChildren().addAll(serverLabel, serverTextField);

        return serverControlVBox;
    }


    private HBox userControlPane() {
        userControlVBox = new HBox();
        userControlVBox.setSpacing(10);
        userControlVBox.setAlignment(Pos.CENTER_LEFT);

        Label usernameLabel = new Label("用户名：");

        String username = DEFAULT_USERNAMES[new Random().nextInt(DEFAULT_USERNAMES.length)];
        usernameTextField = new TextField(username);
        usernameTextField.setPrefWidth(200);

        loginButton = new Button("登录");
        loginButton.setOnAction(e -> {
            try {
                this.auth(Constant.OPCODE_LOGIN);

                logoutButton.setDisable(false);
                loginButton.setDisable(true);
                serverTextField.setDisable(true);
                usernameTextField.setDisable(true);
                messageControlVBox.setDisable(false);
            } catch (Exception ex) {
                ex.printStackTrace();
                messageTextArea.textProperty().set("登录服务器时发生错误：" + ex);
            }
        });

        logoutButton = new Button("注销");
        logoutButton.setDisable(true);
        logoutButton.setOnAction(e -> {
            try {
                this.auth(Constant.OPCODE_LOGOUT);

                logoutButton.setDisable(true);
                loginButton.setDisable(false);
                serverTextField.setDisable(false);
                usernameTextField.setDisable(false);
                messageControlVBox.setDisable(true);
            } catch (Exception ex) {
                ex.printStackTrace();
                messageTextArea.textProperty().set("登录服务器时发生错误：" + ex);
            }
        });

        userControlVBox.getChildren().addAll(usernameLabel, usernameTextField, loginButton, logoutButton);

        return userControlVBox;
    }

    private void auth(byte[] opcode) throws Exception {
        String[] address = serverTextField.getText().split(":");
        int port = Integer.parseInt(address[1]);

        ClientConnection connection = new ClientConnection(address[0], port);

        byte[] data = usernameTextField.getText().getBytes(StandardCharsets.UTF_8);
        OutputStream outputStream = connection.getOutputStream();

        Common.send(data, opcode, outputStream);
        connection.finishOutput();

        InputStream inputStream = connection.getInputStream();
        byte[] header = new byte[4];
        inputStream.read(header);

        DataPack dataPack = Common.readDataPack(inputStream);
        MessageData messageData = MessageData.parse(dataPack.getData());
        messageTextArea.textProperty().set(messageTextArea.getText() +
                        String.format("\n服务器[%s]响应：%s", messageData.getIdText(), messageData.getMessageText()));
        connection.finishInput();

        connection.disconnect();
    }

    private HBox messageControlPane() {
        messageControlVBox = new HBox();
        messageControlVBox.setSpacing(10);
        messageControlVBox.setAlignment(Pos.CENTER_LEFT);
        messageControlVBox.setDisable(true);

        Label usernameLabel = new Label("输入消息：");

        messageTextField = new TextField();
        messageTextField.setPrefWidth(320);

        sendButton = new Button("发送");
        sendButton.setOnAction(e -> {
            String[] address = serverTextField.getText().split(":");
            int port = Integer.parseInt(address[1]);
            try {
                ClientConnection connection = new ClientConnection(address[0], port);

                MessageData messageData = new MessageData(usernameTextField.getText(), messageTextField.getText());

                Common.send(messageData.toBytes(), Constant.OPCODE_SEND_MESSAGE, connection.getOutputStream());
                connection.finishOutput();

                InputStream inputStream = connection.getInputStream();
                byte[] header = new byte[4];
                inputStream.read(header);

                DataPack dataPack = Common.readDataPack(inputStream);
                MessageData responseData = MessageData.parse(dataPack.getData());
                messageTextArea.textProperty().set(messageTextArea.getText() +
                        String.format("\n服务器[%s]响应：%s", responseData.getIdText(), responseData.getMessageText()));
                connection.finishInput();

                connection.disconnect();
            } catch (IOException ex) {
                ex.printStackTrace();
                messageTextArea.textProperty().set("登录服务器时发生错误：" + ex);
            }
        });

        messageControlVBox.getChildren().addAll(usernameLabel, messageTextField, sendButton);

        return messageControlVBox;
    }
}